package com.ddj.uddi;

import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.Enumeration;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.messaging.URLEndpoint;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException ;
import javax.xml.soap.SOAPMessage;

public class RegistryServerV1 extends HttpServlet {
    
    private static final int SEND_XML = 1;
    private static final int SEND_HTML = 2;
    
    protected void processRequest(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        
        String queryString = request.getParameter("queryString");
        String nextJSP = null;
        String url = null;
        if ("FIND".equals(request.getParameter("submitPredefined"))) {
            url = request.getParameter("predefinedURL");
        }
        else if ("FIND".equals(request.getParameter(
                    "submitUserSpecified"))) {
            url = request.getParameter("userSpecifiedURL");
        }
        if (queryString.length() == 0) {
            nextJSP = "/MissingSearchString.jsp";
        }
        else if (url == null) {
            nextJSP = "/MissingURL.jsp";
        }
        else {
            try {
                SOAPMessage reply = findBusiness(url, queryString);
                int disp = Integer.parseInt(request.getParameter(
                    "disposition"));
                switch (disp) {
                    case SEND_XML:
                        response.setContentType("text/xml");
                        OutputStream os = response.getOutputStream();
                        reply.writeTo(os);
                        os.flush();
                        os.close();
                        break;
                    case SEND_HTML:
                        transformMessage(reply, "BusinessList1.xsl",
                            response);
                        break;
                    default:
                        throw new ServletException("error");
                }
                
            }
            catch (Exception e) {
                e.printStackTrace();
                throw new ServletException("unable to send query");
            }
        }
        if (nextJSP != null) {
            RequestDispatcher rd = 
                getServletConfig().getServletContext()
                .getRequestDispatcher(nextJSP);
            rd.forward(request,response);
        }
    }
    
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    protected void doPost(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    public String getServletInfo() {
        return "Registry Servlet V1";
    }
    
    private SOAPMessage findBusiness(String url, String queryString)
    throws Exception {
        MessageFactory msgFactory = MessageFactory.newInstance();
        SOAPMessage msg = msgFactory.createMessage();
        SOAPEnvelope envelope = msg.getSOAPPart().getEnvelope();
        SOAPBody body = envelope.getBody();
        SOAPElement findBusiness = body.addBodyElement(
            envelope.createName("find_business",
            "", "urn:uddi-org:api"));
        findBusiness.addAttribute(
            envelope.createName("generic"), "1.0");
        findBusiness.addAttribute(
            envelope.createName("maxRows"), "100");
        SOAPElement businessName =
            findBusiness.addChildElement(
            envelope.createName("name"));
        businessName.addTextNode(queryString);
        msg.saveChanges();
                SOAPConnectionFactory scf = 
            SOAPConnectionFactory.newInstance();
        SOAPConnection connection = scf.createConnection();
        URLEndpoint endpoint = new URLEndpoint(url);
        SOAPMessage reply = connection.call(msg, endpoint);
        return reply;
    }
    
    private void transformMessage(SOAPMessage msg, String xslSource,
    HttpServletResponse response)
    throws Exception {
        response.setContentType("text/html");
        ServletOutputStream out = response.getOutputStream();
        ServletContext context = getServletConfig().
            getServletContext();
        String filePath = context.getRealPath(xslSource);
        PipedOutputStream pipeToThread = new PipedOutputStream();
        PipedInputStream pipeFromThread = new PipedInputStream();
        TransformerThread transformer =
        new TransformerThread(filePath, pipeToThread, pipeFromThread);
        new Thread(transformer).start();
        msg.writeTo(pipeToThread);
        pipeToThread.flush();
        pipeToThread.close();
        int b;
        while ((b = pipeFromThread.read()) >= 0) {
            out.print((char)b);
        }
        pipeFromThread.close();
        out.flush();
        out.close();
    }
}
