package com.ddj.uddi;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.messaging.URLEndpoint;
import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.FindQualifier;
import javax.xml.registry.JAXRException;
import javax.xml.registry.RegistryService;
import javax.xml.registry.infomodel.EmailAddress;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.PersonName;
import javax.xml.registry.infomodel.RegistryObject;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;
import javax.xml.registry.infomodel.TelephoneNumber;
import javax.xml.registry.infomodel.User;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException ;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

public class RegistryServerV2 extends HttpServlet {
    
    private static final int SEND_XML = 1;
    private static final int SEND_HTML = 2;
        
    protected void processRequest(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        
        String queryString = request.getParameter("queryString");
        String nextJSP = null;
        String url = null;
        if ("FIND".equals(request.getParameter("submitPredefined"))) {
            url = request.getParameter("predefinedURL");
        }
        else if ("FIND".equals(request.getParameter(
        "submitUserSpecified"))) {
            url = request.getParameter("userSpecifiedURL");
        }
        if (queryString.length() == 0) {
            nextJSP = "/MissingSearchString.jsp";
        }
        else if (url == null) {
            nextJSP = "/MissingURL.jsp";
        }
        else {
            try {
                SOAPMessage reply = findBusiness(url, queryString);
                int disp = Integer.parseInt(request.getParameter(
                    "disposition"));
                switch (disp) {
                    case SEND_XML:
                        response.setContentType("text/xml");
                        OutputStream os = response.getOutputStream();
                        reply.writeTo(os);
                        os.flush();
                        os.close();
                        break;
                    case SEND_HTML:
                        transformMessage(reply, "BusinessList2.xsl",
                            response);
                        break;
                    default:
                        throw new ServletException("error");
                }
                
            }
            catch (Exception e) {
                e.printStackTrace();
                throw new ServletException("unable to send query");
            }
        }
        if (nextJSP != null) {
            RequestDispatcher rd = 
                getServletConfig().getServletContext()
                .getRequestDispatcher(nextJSP);
            rd.forward(request,response);
        }
    }
    
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    protected void doPost(HttpServletRequest request,
            HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    public String getServletInfo() {
        return "Registry Servlet V2";
    }
    
    private SOAPMessage findBusiness(String url, String queryString)
    throws Exception {
        BulkResponse response = null;
        Connection connection = null;
        SOAPMessage msg = null;
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL",
            url);
        props.setProperty("javax.xml.registry.factoryClass",
            "com.sun.xml.registry.uddi.ConnectionFactoryImpl");
        try {
            ConnectionFactory factory =
            ConnectionFactory.newInstance();
            factory.setProperties(props);
            connection = factory.createConnection();
            RegistryService rs = connection.getRegistryService();
            BusinessQueryManager bqm =
            rs.getBusinessQueryManager();
            Collection findQualifiers = new ArrayList();
            findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
            Collection namePatterns = new ArrayList();
            namePatterns.add(queryString + "%");
            
            response = bqm.findOrganizations(findQualifiers,
            namePatterns, null, null, null, null);
            msg = convertToSOAPMessage(response);
        }
        catch (Exception e) {
            e.printStackTrace();
        } finally  {
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
            return msg;
        }
    }
    
    private SOAPMessage convertToSOAPMessage(BulkResponse response) 
        throws Exception {
        MessageFactory factory = MessageFactory.newInstance();
        SOAPMessage msg = factory.createMessage();
        SOAPEnvelope envelope = msg.getSOAPPart().getEnvelope();
        SOAPBody body = envelope.getBody();
        Collection orgs = response.getCollection();
        Iterator orgIter = orgs.iterator();
        while (orgIter.hasNext()) {
            Organization org =
                (Organization) orgIter.next();
            SOAPBodyElement orgElement = body.addBodyElement(
                envelope.createName("Organization"));
            SOAPElement orgName = orgElement.addChildElement(
                envelope.createName("name"));
            orgName.addTextNode(getName(org));
            SOAPElement orgKey = orgElement.addChildElement(
                envelope.createName("key"));
            orgKey.addTextNode(getKey(org));
            SOAPElement orgDesc = orgElement.addChildElement(
                envelope.createName("description"));
            orgDesc.addTextNode(getDescription(org));
        }
        msg.saveChanges();
        return msg;
    }
    
    private void transformMessage(SOAPMessage msg, 
        String xslSource, HttpServletResponse response)
        throws Exception {
        response.setContentType("text/html");
        ServletOutputStream out = response.getOutputStream();
        ServletContext context = getServletConfig().
                getServletContext();
        String filePath = context.getRealPath(xslSource);
        PipedOutputStream pipeToThread = new PipedOutputStream();
        PipedInputStream pipeFromThread = new PipedInputStream();
        TransformerThread transformer =
        new TransformerThread(filePath, pipeToThread, pipeFromThread);
        new Thread(transformer).start();
        msg.writeTo(pipeToThread);
        pipeToThread.flush();
        pipeToThread.close();
        int b;
        while ((b = pipeFromThread.read()) >= 0) {
            out.print((char)b);
        }
        pipeFromThread.close();
        out.flush();
        out.close();
    }
    
    private String getName(RegistryObject ro) throws JAXRException {
        try {
            return ro.getName().getValue();
        } catch (NullPointerException npe) {
            return "";
        }
    }
    
    private String getDescription(RegistryObject ro) 
    throws JAXRException {
        try {
            return ro.getDescription().getValue();
        } catch (NullPointerException npe) {
            return "";
        }
    }
    
    private String getKey(RegistryObject ro) throws JAXRException {
        try {
            return ro.getKey().getId();
        } catch (NullPointerException npe) {
            return "";
        }
    }
}
