module TaskMaster
  module Mixins
    ##################################################################
    # ClientAttributes module
    # Here is where we specify methods which return various attributes
    # about the client.  Users who wish to check different attributes
    # of their clients (which are not defined here) should open up
    # the ClientAttributes module in a different file and add the
    # desired methods.  This module is mixed-in to Client (see Client.rb)
    # so the user added methods will get mixed-in as well.
    # Phil Tomson 1/17/2002
    # Copyright (C) 2002 Phil Tomson
    # This file covered by the same license as Ruby's
    ##################################################################
    module ClientAttributes
    ##################################################################
    # cwd - return the current working directory
    ##################################################################
       def cwd
         puts "Client::cwd() " if $DEBUG
         Dir.pwd
       end
    ##################################################################
    # login - return the current login for this machine
    ##################################################################
       def login
          puts "Client::login() " if $DEBUG
	  case RUBY_PLATFORM
	  when /win/i, /ming/i
	    #Windows
	    require 'Win32API'
            getUserName = Win32API.new("advapi32","GetUserName",['P','P'],'L')
            buffer = "\0" * 256;
            nsize =  [256].pack("L")
            getUserName.Call(buffer,nsize)
            len = nsize.unpack("L")[0]
            username = buffer[0 ... len].chop
	  when /ix/i, /ux/i, /gnu/i, /solaris/i, /sunos/i, /bsd/i
	    require 'etc'
            Etc.getlogin
	  end
       end

    ##################################################################
    # osType - return RUBY_PLATFORM for this machine
    ##################################################################
       def osType
          return RUBY_PLATFORM #platform this ruby was built for
       end

    ##################################################################
    # rubyVer - return RUBY_VERSION (version number of Ruby running)
    ##################################################################
       def rubyVer
          return RUBY_VERSION
       end

    end #ClientAttributes
  end #Mixins module
end #TaskMaster module
