####################################################################
# Slave.rb
# Defines the Slave class which runs as a dRuby object on a slave 
# machine.
# also provides Client class for backwards compatibility
# 12/12/2001
# Copyright (C) 2001 Phil Tomson
# This file covered by the same license as Ruby's
# See 'COPYING' file
####################################################################
require "timeout"
require "TaskMaster"
module TaskMaster

  class Slave
     include Mixins::RemoteRequirer
     include Mixins::ClientAttributes
     attr_writer :clientTimeLimit
     def initialize(port = DEFAULT_PORT,timelimit = 99999)#assume an 'infinite' timelimit for default
        @port = port
        @current_task = nil
        @clientTimeLimit= timelimit 
	#TODO: Why not start the DRb service in here?
        DRb.start_service("druby://0.0.0.0:#{@port}",self)
     end
     #you need to call this at the end of your slave script
     def wait 
        DRb.thread.join
     end
    ##################################################################
    # run - 'run' the object that was sent here
    #       task is either:
    #          - a String (in which case we try to eval it,unless it has args)
    #          - or some other user-defined type of object (Task)
    #            which must respond to the 'run' method
    #       clientTimeLimit is the client-side timeout
    ##################################################################
     def run(task,args) #TODO:do we really need to have *args here?
        @current_task = task
        puts "In Client::run(#{task.type},#{args})" if $DEBUG
        result = ""
        puts "Note: @clientTimeLimit is: #@clientTimeLimit" if $DEBUG
        begin
           if( task.respond_to?(:run))
              #initialize the task object in the context of the client:
              task.pre_run_init if task.respond_to?(:pre_run_init)
	      if args != nil #not nil (we wanna be able to pass 'false')
                 result = task.run(*args)
	      else #args is nil
		 result = task.run()
	      end
              task.post_run_cleanup if task.respond_to?(:post_run_cleanup)
           else
              puts "task type is: #{task.type} has no 'run' method"
              result = "COULD_NOT_RUN" 
           end
        rescue TimeoutError
           puts "Client::run - CLIENT_TIMED_OUT"
           result = "CLIENT_TIMED_OUT"
        rescue #other types of errors which might be generated by the above
      
        end 
        return result 
     end #run

     def harvest
        if @current_task.respond_to?(:harvest)
          return @current_task.harvest 
        end
     end

     def add_method(methodAsStr)
        self.instance_eval(methodAsStr)
     end

     ##############################################
     # query is primarily used to check to see
     # if the drb connection is OK
     ##############################################
     def query
       #see ClientAttributes.rb for definitions of these functions:
       return cwd(),login(),osType(),rubyVer()
     end

  end #class Slave

  class Client < Slave
    #provided for backwards compatibility 
     include Mixins::RemoteRequirer
     include Mixins::ClientAttributes
  end

end #TaskMaster module



if $0 == __FILE__
   require 'TaskMaster'
   clientObj = TaskMaster::Client.new(9876)
   #DRb.start_service('druby://0.0.0.0:9000',clientObj)
   puts "Client started"
   clientObj.wait

end
