####################################################################
# SlaveProxy.rb
# Defines the SlaveProxy class which acts as a local proxy for
# the remote Slave object. (note: ClientProxy is defined for 
# the sake sake of backward compatibility)
# 12/12/2001
# Copyright (C) 2001 Phil Tomson
# This file covered by the same license as Ruby's
# See 'COPYING' file
####################################################################
module TaskMaster
  require 'drb'
  require "common.rb"
  #require 'etc' #shouldn't need this anymore
  require "timeout"
  require "thread"
  #########################################################################
  # SlaveProxy - responsible for actual connection to a slave machine
  # also maintains a list of slave attributes
  # keeps track of slave machine's status (is it Available, Busy, Finished
  # or has it TimedOut )
  #########################################################################
  class SlaveProxy
     attr_reader :name, :osType, :rubyVer, :cwd, :login, :status
     attr_writer :serverTimeLimit
     def initialize(name,port=DEFAULT_PORT)
        #client attributes
        @name   = name
	@port   = port
        @status = "UNKNOWN" #later, one of: AVAILABLE,BUSY,FINISHED,TIMED_OUT
        @drbObj = nil
        @serverTimeLimit = 9999999 #default to 'infinite'
        #establish the DrubyObject for this client
        begin
           puts "trying: druby://#{name}:#{port}"  if $DEBUG
           @drbObj = DRbObject.new(nil, "druby://#{name}:#@port")
        rescue
           raise #reraise exception for Distributor to deal with
        else
           dbg_puts "ClientProxy::initialize(#{name}):dRuby connection established"
           #now get information about the client (wrap in a timeout just in case)
	   begin
	      timeout(20) do #20 seconds should be sufficient to get these 
	         cwd,login,osType,rubyVer = @drbObj.query
	         puts "Attributes for #{name} : "
	         puts "   cwd:    #{cwd}"
	         puts "   login:  #{login}"
	         puts "   OS:     #{osType}"
	         puts "   ruby version: #{rubyVer}" 
	      end #timeout
	   rescue TimeoutError
	      puts "Could not complete attribute query for #{name}"
	      raise #let Distributor know about this
	   rescue 
	      raise #let Distributor know about this
	   else
	     @status = "AVAILABLE" 
	   end
        end
     end #initialize

     def check_attributes(attribsToCheck)
        return unless attribsToCheck
        str = ""
        attribsToCheck.each { |attrib,strToMatch|
	   clientAttrib = ""
	   if strToMatch
	      begin
	         timeout(20) {#20 seconds should be sufficient
	            clientAttrib = @drbObj.send(attrib).strip
	         }
	      rescue TimeoutError
	         puts "could not complete attribute query for #@name"
	         raise
	      end
	      unless clientAttrib == strToMatch.strip
	         str << "#{attrib} is: #{clientAttrib} and doesn't match #{strToMatch}\a\n"
	      end
           end
        }
        raise AttributeMismatch, str+" on client: #@name\a\a" if str.length > 0
     end

     def isAvailable?
        return true if @status == "AVAILABLE"
     end

     def run(whatToRun, args)
        # args should be an array at this point
	# to make it easier for the user, if there's only one item in 
	# the args array let's extract that item and make args equal to it
	# so that the user doesn't have to do something like:
	# something = args[0]
        args = args[0] if args.length == 1	
        puts "Note: @serverTimeLimit is: #@serverTimeLimit " if $DEBUG
	puts "args type is: #{args.type}" 
        dbg_puts "ClientProxy::run(#{whatToRun.type})"
        @thread = Thread.start do
          @status = "BUSY"
          begin
            timeout(@serverTimeLimit) do
               #@status = "BUSY"
               puts "In timeout:ClientProxy::run - #@name's status is: #@status"
               @drbObj.run(whatToRun,args)
            end #timeout
          rescue TimeoutError #server-side timeout-assume client is dead
            @status = "TIMED_OUT"
            puts "rescue:ClientProxy::run - #@name's status is: #@status"
            @drbObj = nil #break the connection
          #NOTE: do we want to 'rescue' any other types of exceptions here?
          else #all was well
             @status = "FINISHED" #unless @thread.alive?#completed task before timeout
            puts "else:ClientProxy::run - #@name's status is: #@status"
             #NOTE: now that status is FINISHED, it means that the
             #results of running the last task need to be harvested from
             #the client object before it is AVAILABLE again - Distributor
             #initiates the 'harvesting' when it sees that the status of
             #a given ClientProxy/Client is FINISHED
          end 
        end #Thread

            puts "After Thread:ClientProxy::run - #@name's status is: #@status"
     end #run

     def get_status
        #if status was BUSY, but now the thread is no longer alive:
        if (@status == "BUSY" && !(@thread.alive?) )
           @status = "FINISHED"
        end
        @status
     end

     def harvest(reporterObj=nil)
        puts "ClientProxy::harvest(#{reporterObj})" if $DEBUG

        #status was FINISHED and now becomes AVAILABLE
        @status = "AVAILABLE"
        @thread = nil
        puts "ClientProxy::harvest reporterObj.type = #{reporterObj.type}"
        array = @drbObj.harvest
	if array
          reporterObj.report(*array) if reporterObj
        end
     end

     def join_thread
        puts "ClientProxy::join_thread" if $DEBUG
        @thread.join if @thread
     end

     #pass on any methods we don't know about to the client
     def method_missing(methID, *args)
        @drbObj.send(methID, *args) 
     end

  end #class ClientProxy

  class ClientProxy < SlaveProxy
  end

end #TaskMaster module

