#Task.rb
#used for testing Distributor
#Notice how each of the classes in this file defines a 'run' method - 
#this makes them 'runnable'.  Objects which are 'runnable' by this definition
#can be distributed to clients by the Distributor 

require 'common.rb'
require 'drb'
#Task - this is for testing purposes only
class Task
   def initialize(times = 5)
      @times = times
   end
   
   def run(*args)
      puts "Task::Run - running #{args}"
      @times.times do
         print "."
         $stdout.flush
         sleep 1 
      end
      puts "\nTask::Run - end #{args}"
      return "OK"
   end
end

#ChildTask - simulates a case in which a tool might be run multiple times
#on the client - note that in this case serverTimeLimit and clientTimeLimit
#will be different (this feature is for backward compatibility with 
#older versions of dTest - You may not need this feature.
class ChildTask < Task
   def initialize(sleeptime = 5, number_of_times = 2)
      @sleeptime = sleeptime
      @number_of_times = number_of_times
      super(sleeptime)
   end

   def run(*args)
      puts "ChildTask::Run"
      #super(args)
      @number_of_times.times { super(args) }
      return "OK"
   end

   def get_clientTimeLimit
      return (@sleeptime + 1)
   end
   def get_serverTimeLimit
      return ((@sleeptime +1) * @number_of_times)
   end
end

class InitClass
   def initialize(message)
      @message = message
   end
   def run(*args)
      puts @message
      return "OK"
   end
end


#for testing Task with one client specified on the command line
if $0 == __FILE__
   
   if ARGV.length == 0
      raise "You need to specify a machine name or address as a commandline arg"
   end
   DRb.start_service()
   client = DRbObject.new(nil,"druby://#{ARGV.shift}:9000")
   client.remote_require(File.new($0).stringify) #tell it about this class
   puts client.run(Task.new, "first")
   puts client.run(Task.new, "second")
   puts client.run(Task.new, "third")
  # puts client.run(ChildTask.new, "Wow!")
   puts client.run  "puts \"here we go!\""
   #puts client.add_method( :print_message ) {puts "print this informative message"}
   #puts client.print_message
   #now locally:
   Task.new.run("local1")
   Task.new.run("local2")
end
