require "../TM_finder"
require "TaskMaster"
require "PiTask"
require 'getoptlong' #for command line args handling
opts = GetoptLong.new(
[ "--slaves","-s",			GetoptLong::REQUIRED_ARGUMENT ],
[ "--iterations","-i",			GetoptLong::REQUIRED_ARGUMENT ],
[ "--help","-h",		        GetoptLong::NO_ARGUMENT ]
)
slaveList = []
iterations = 1000000 #by default
opts.each do |opt,arg|
  case opt
  when "--help","-h"
    puts "ruby master.rb <options>"
    puts "options are:"
    puts "\t--slaves, -s : comma seperated list of slave machines"
    puts "\t--iterations, -i : number of iterations to run"
    puts "\t--help, -h    : displays this helpful message"
    exit
  when "--slaves","-s"
    slaveList = arg.split(/,/)
  when "--iterations","-i"
    iterations = arg.to_i
  else
    puts "invalid option: #{opt}"
    exit
  end
end

class PiReporter
  def initialize
    @estimates = []
  end

  def report(pi_est)
    @estimates << pi_est
  end

  def final_report
    sum = 0
    @estimates.each { |est|
      sum += est
    }
    print "The average pi estimate was: "
    puts "#{sum/(@estimates.length)}"
  end
end
startTime = Time.now
filesToRequire = ["PiTask.rb"]
filesToRequire.each { |file| require file }
distrib = TaskMaster::Distributor.new(slaveList)
distrib.remote_require(filesToRequire)
reporter = PiReporter.new
distrib.reporter = reporter
(distrib.availableSlaves).each {
  distrib.send_task(PiTask.new(),(iterations/distrib.availableSlaves.length))
}
distrib.wait
reporter.final_report
endTime = Time.now
puts "Total time: #{endTime-startTime} seconds"
